﻿using MapControl;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace GWXDeliveryDemo
{
    public class RouteSegmentVisualInfo
    {
        readonly Map _mapControl;
        MapPolyline _routePolyline;
        readonly LocationCollection _points;
        readonly Color _ordinaryColor = Colors.Blue;
        readonly Color _highlightedColor = Colors.Yellow;
        private const double OrdinaryOpacity = 0.5;
        private const double HighlightedOpacity = 1;
        Pushpin _tooltipPushpin;
        readonly string _tooltipText;

        public RouteSegmentVisualInfo(Map mapControl, IEnumerable<Location> points, string tooltipText)
        {
            _mapControl = mapControl;
            _points = new LocationCollection(points);
            _tooltipText = tooltipText;

            InitPolyline();
            InitTooltip();
            InitEvents();
            Invalidate();
        }

        public void Clear()
        {
            _mapControl.Children.Remove(_tooltipPushpin);
            _mapControl.Children.Remove(_routePolyline);
        }

        void InitPolyline()
        {
            _routePolyline = new MapPolyline {Locations = _points, IsClosed = false, StrokeThickness = 5};
            SetOrdinaryBrush();
        }

        void InitTooltip()
        {
            _tooltipPushpin = new Pushpin {Visibility = Visibility.Collapsed};
            Panel.SetZIndex(_tooltipPushpin, 2);
            _mapControl.Children.Add(_tooltipPushpin);
        }

        void InitEvents()
        {
            _routePolyline.MouseEnter += (s, e) =>
            {
                var pos = e.GetPosition(_mapControl);
                var location = _mapControl.ViewportPointToLocation(pos);
                ShowTooltip(location);
                SetHighlightBrush();
                Invalidate();
            };

            _routePolyline.MouseLeave += (s, e) =>
            {
                HideTooltip();
                SetOrdinaryBrush();
                Invalidate();
            };
        }

        void Invalidate()
        {
            _mapControl.Children.Remove(_routePolyline);
            _mapControl.Children.Add(_routePolyline);
        }

        void ShowTooltip(Location location)
        {
            MapPanel.SetLocation(_tooltipPushpin, location);
            _tooltipPushpin.Content = _tooltipText;
            _tooltipPushpin.Visibility = Visibility.Visible;
        }

        void HideTooltip()
        {
            _tooltipPushpin.Visibility = Visibility.Collapsed;
        }

        void SetOrdinaryBrush()
        {
            SetBrush(_ordinaryColor, OrdinaryOpacity);
        }

        void SetHighlightBrush()
        {
            SetBrush(_highlightedColor, HighlightedOpacity);
        }

        void SetBrush(Color color, double opacity)
        {
            var lineBrush = new SolidColorBrush {Color = color, Opacity = opacity};
            _routePolyline.Stroke = lineBrush;
        }
    }
}
