﻿using System;
using MapControl;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace GWXDeliveryDemo
{
    public class RouteStopVisualInfo
    {
        readonly Map _mapControl;
        Path _routeStop;
        Pushpin _tooltipPushpin;
        string _tooltipText;
        readonly Location _location;

        readonly Color _ordinaryColor = Colors.Green;
        readonly Color _highlightedColor = Colors.Yellow;
        
        public RouteStopVisualInfo(Map mapControl, Location point, string tooltipText)
        {
            _mapControl = mapControl;
            _tooltipText = tooltipText;
            _location = point;

            InitPath(point);
            InitTooltip();
            InitEvents();
            Invalidate();
        }

        public bool IsSamePoint(Location point)
        {
            return Math.Abs(_location.Latitude - point.Latitude) < float.Epsilon && Math.Abs(_location.Longitude - point.Longitude) < float.Epsilon;
        }

        public void AddTooltipText(string text)
        {
            _tooltipText += "\n-----------------\n";
            _tooltipText += text;
            _tooltipPushpin.Content = _tooltipText;
        }

        public void Clear()
        {
            _mapControl.Children.Remove(_tooltipPushpin);
            _mapControl.Children.Remove(_routeStop);
        }

        void InitPath(Location point)
        {
            _routeStop = new Path
            {
                Data = new EllipseGeometry
                {
                    RadiusX = 75,
                    RadiusY = 75,
                    Transform = _mapControl.ScaleTransform
                }
            };
            MapPanel.SetLocation(_routeStop, point);
            Panel.SetZIndex(_routeStop, 1);
            SetOrdinaryBrush();
        }

        void InitTooltip()
        {
            _tooltipPushpin = new Pushpin {Visibility = Visibility.Collapsed};
            Panel.SetZIndex(_tooltipPushpin, 2);
            MapPanel.SetLocation(_tooltipPushpin, _location);
            _tooltipPushpin.Content = _tooltipText;
            _mapControl.Children.Add(_tooltipPushpin);
        }

        void InitEvents()
        {
            _routeStop.MouseEnter += (a, e) =>
            {
                ShowTooltip();
                SetHighlightBrush();
                Invalidate();
            };

            _routeStop.MouseLeave += (s, e) =>
            {
                HideTooltip();
                SetOrdinaryBrush();
                Invalidate();
            };
        }

        void Invalidate()
        {
            _mapControl.Children.Remove(_routeStop);
            _mapControl.Children.Add(_routeStop);
        }

        void SetOrdinaryBrush()
        {
            SetBrush(_ordinaryColor);
        }

        void SetHighlightBrush()
        {
            SetBrush(_highlightedColor);
        }

        void SetBrush(Color color)
        {
            var brush = new SolidColorBrush {Color = color};
            _routeStop.Stroke = brush;
            _routeStop.Fill = brush;
            _routeStop.Opacity = 0.5;
        }

        void ShowTooltip()
        {
            _tooltipPushpin.Visibility = Visibility.Visible;
        }

        void HideTooltip()
        {
            _tooltipPushpin.Visibility = Visibility.Collapsed;
        }
    }
}
